@@ -1,23 +1,17 @@
 /*
  * Copyright (C) 2016 Google, Inc.
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 #include <array>
@@ -74,7 +68,8 @@ void Smoke::init_workers()
         worker_count = 1;
     }
 
-    const int object_per_worker = sim_.objects().size() / worker_count;
+    const int object_per_worker =
+        static_cast<int>(sim_.objects().size()) / worker_count;
     int object_begin = 0, object_end = 0;
 
     workers_.reserve(worker_count);
@@ -83,7 +78,7 @@ void Smoke::init_workers()
         if (i < worker_count - 1)
             object_end += object_per_worker;
         else
-            object_end = sim_.objects().size();
+            object_end = static_cast<int>(sim_.objects().size());
 
         Worker *worker = new Worker(*this, i, object_begin, object_end);
         workers_.emplace_back(std::unique_ptr<Worker>(worker));
@@ -388,11 +383,6 @@ void Smoke::destroy_frame_data()
     if (!use_push_constants_) {
         vk::DestroyDescriptorPool(dev_, desc_pool_, nullptr);
 
-        for (auto cmd_pool : worker_cmd_pools_)
-            vk::DestroyCommandPool(dev_, cmd_pool, nullptr);
-        worker_cmd_pools_.clear();
-        vk::DestroyCommandPool(dev_, primary_cmd_pool_, nullptr);
-
         vk::UnmapMemory(dev_, frame_data_mem_);
         vk::FreeMemory(dev_, frame_data_mem_, nullptr);
 
@@ -400,6 +390,11 @@ void Smoke::destroy_frame_data()
             vk::DestroyBuffer(dev_, data.buf, nullptr);
     }
 
+    for (auto cmd_pool : worker_cmd_pools_)
+        vk::DestroyCommandPool(dev_, cmd_pool, nullptr);
+    worker_cmd_pools_.clear();
+    vk::DestroyCommandPool(dev_, primary_cmd_pool_, nullptr);
+
     for (auto &data : frame_data_)
         vk::DestroyFence(dev_, data.fence, nullptr);
 
@@ -471,7 +466,7 @@ void Smoke::create_buffers()
         object_data_size += alignment - (object_data_size % alignment);
 
     // update simulation
-    sim_.set_frame_data_size(object_data_size);
+    sim_.set_frame_data_size(static_cast<uint32_t>(object_data_size));
 
     VkBufferCreateInfo buf_info = {};
     buf_info.sType = VK_STRUCTURE_TYPE_BUFFER_CREATE_INFO;
@@ -525,11 +520,11 @@ void Smoke::create_descriptor_sets()
 {
     VkDescriptorPoolSize desc_pool_size = {};
     desc_pool_size.type = VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC;
-    desc_pool_size.descriptorCount = frame_data_.size();
+    desc_pool_size.descriptorCount = static_cast<uint32_t>(frame_data_.size());
 
     VkDescriptorPoolCreateInfo desc_pool_info = {};
     desc_pool_info.sType = VK_STRUCTURE_TYPE_DESCRIPTOR_POOL_CREATE_INFO;
-    desc_pool_info.maxSets = frame_data_.size();
+    desc_pool_info.maxSets = static_cast<uint32_t>(frame_data_.size());
     desc_pool_info.poolSizeCount = 1;
     desc_pool_info.pPoolSizes = &desc_pool_size;
 
@@ -782,18 +777,21 @@ void Smoke::on_frame(float frame_pred)
 
     VkResult res = vk::BeginCommandBuffer(data.primary_cmd, &primary_cmd_begin_info_);
 
-    VkBufferMemoryBarrier buf_barrier = {};
-    buf_barrier.sType = VK_STRUCTURE_TYPE_BUFFER_MEMORY_BARRIER;
-    buf_barrier.srcAccessMask = VK_ACCESS_HOST_WRITE_BIT;
-    buf_barrier.dstAccessMask = VK_ACCESS_SHADER_READ_BIT;
-    buf_barrier.srcQueueFamilyIndex = VK_QUEUE_FAMILY_IGNORED;
-    buf_barrier.dstQueueFamilyIndex = VK_QUEUE_FAMILY_IGNORED;
-    buf_barrier.buffer = data.buf;
-    buf_barrier.offset = 0;
-    buf_barrier.size = VK_WHOLE_SIZE;
-    vk::CmdPipelineBarrier(data.primary_cmd,
-            VK_PIPELINE_STAGE_HOST_BIT, VK_PIPELINE_STAGE_VERTEX_SHADER_BIT,
-            0, 0, nullptr, 1, &buf_barrier, 0, nullptr);
+    if (!use_push_constants_) {
+        VkBufferMemoryBarrier buf_barrier = {};
+        buf_barrier.sType = VK_STRUCTURE_TYPE_BUFFER_MEMORY_BARRIER;
+        buf_barrier.srcAccessMask = VK_ACCESS_HOST_WRITE_BIT;
+        buf_barrier.dstAccessMask = VK_ACCESS_SHADER_READ_BIT;
+        buf_barrier.srcQueueFamilyIndex = VK_QUEUE_FAMILY_IGNORED;
+        buf_barrier.dstQueueFamilyIndex = VK_QUEUE_FAMILY_IGNORED;
+        buf_barrier.buffer = data.buf;
+        buf_barrier.offset = 0;
+        buf_barrier.size = VK_WHOLE_SIZE;
+        vk::CmdPipelineBarrier(data.primary_cmd,
+                               VK_PIPELINE_STAGE_HOST_BIT,
+                               VK_PIPELINE_STAGE_VERTEX_SHADER_BIT,
+                               0, 0, nullptr, 1, &buf_barrier, 0, nullptr);
+    }
 
     render_pass_begin_info_.framebuffer = framebuffers_[back.image_index];
     render_pass_begin_info_.renderArea.extent = extent_;
