@@ -5,24 +5,17 @@
  * Copyright (c) 2015-2016 LunarG, Inc.
  * Copyright (C) 2016 Google Inc.
  *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and/or associated documentation files (the "Materials"), to
- * deal in the Materials without restriction, including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
- * sell copies of the Materials, and to permit persons to whom the Materials are
- * furnished to do so, subject to the following conditions:
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
- * The above copyright notice(s) and this permission notice shall be included in
- * all copies or substantial portions of the Materials.
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * THE MATERIALS ARE PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
- *
- * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
- * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
- * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE MATERIALS OR THE
- * USE OR OTHER DEALINGS IN THE MATERIALS.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  *
  * Author: Courtney Goeltzenleuchter <courtney@lunarg.com>
  * Author: Jon Ashburn <jon@lunarg.com>
@@ -627,26 +620,24 @@ static inline void loader_init_instance_extension_dispatch_table(
             inst, "vkGetPhysicalDeviceXlibPresentationSupportKHR");
 #endif
     table->GetPhysicalDeviceDisplayPropertiesKHR =
-        (PFN_vkGetPhysicalDeviceDisplayPropertiesKHR) gpa(inst,
-        "vkGetPhysicalDeviceDisplayPropertiesKHR");
+        (PFN_vkGetPhysicalDeviceDisplayPropertiesKHR)gpa(
+            inst, "vkGetPhysicalDeviceDisplayPropertiesKHR");
     table->GetPhysicalDeviceDisplayPlanePropertiesKHR =
-        (PFN_vkGetPhysicalDeviceDisplayPlanePropertiesKHR) gpa(inst,
-        "vkGetPhysicalDeviceDisplayPlanePropertiesKHR");
+        (PFN_vkGetPhysicalDeviceDisplayPlanePropertiesKHR)gpa(
+            inst, "vkGetPhysicalDeviceDisplayPlanePropertiesKHR");
     table->GetDisplayPlaneSupportedDisplaysKHR =
-        (PFN_vkGetDisplayPlaneSupportedDisplaysKHR) gpa(inst,
-        "vkGetDisplayPlaneSupportedDisplaysKHR");
-    table->GetDisplayModePropertiesKHR =
-        (PFN_vkGetDisplayModePropertiesKHR) gpa(inst,
-        "vkGetDisplayModePropertiesKHR");
+        (PFN_vkGetDisplayPlaneSupportedDisplaysKHR)gpa(
+            inst, "vkGetDisplayPlaneSupportedDisplaysKHR");
+    table->GetDisplayModePropertiesKHR = (PFN_vkGetDisplayModePropertiesKHR)gpa(
+        inst, "vkGetDisplayModePropertiesKHR");
     table->CreateDisplayModeKHR =
-        (PFN_vkCreateDisplayModeKHR) gpa(inst,
-        "vkCreateDisplayModeKHR");
+        (PFN_vkCreateDisplayModeKHR)gpa(inst, "vkCreateDisplayModeKHR");
     table->GetDisplayPlaneCapabilitiesKHR =
-        (PFN_vkGetDisplayPlaneCapabilitiesKHR) gpa(inst,
-        "vkGetDisplayPlaneCapabilitiesKHR");
+        (PFN_vkGetDisplayPlaneCapabilitiesKHR)gpa(
+            inst, "vkGetDisplayPlaneCapabilitiesKHR");
     table->CreateDisplayPlaneSurfaceKHR =
-        (PFN_vkCreateDisplayPlaneSurfaceKHR) gpa(inst,
-        "vkCreateDisplayPlaneSurfaceKHR");
+        (PFN_vkCreateDisplayPlaneSurfaceKHR)gpa(
+            inst, "vkCreateDisplayPlaneSurfaceKHR");
 }
 
 static inline void *
